/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package core;

import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;

import utils.ImageUtils;

/**
 * Instances of this class represent an clipped image from a original larger image
 * and gives the possibility to transform a path drawn in the clipped image into
 * it's equivalent relative to a smaller version of the original large image.
 */
public class SubImage {

	/**
	 * The clipped image
	 */
	private BufferedImage subImage;

	/**
	 * The transformation that should be made to transform the path
	 */
	private AffineTransform transformToSmallPath;

	/**
	 * Creates a new instance of this class given the small image to compute the
	 * transformations needed to be made to the path created in the clipped image,
	 * the clipping rectangle, the zoom level and the larger
	 * (higher quality) image from where the small image was created
	 * 
	 * @param smallImage the desired image dimensions
	 * @param clipping the rectangle used to do the clipping 
	 * @param zoom how many times the small image to be zoomed
	 * @param largeImage a higher quality image from where the small image was created
	 */
	public SubImage(BufferedImage smallImage, Rectangle2D.Double clipping, int zoom, BufferedImage largeImage) {

		//trace the subImage rectangle in the small image
		Rectangle smallImageCoord = new Rectangle((int)(smallImage.getWidth() * clipping.x),
				(int)(smallImage.getHeight() * clipping.y),
				(int)(smallImage.getWidth() * clipping.width),
				(int)(smallImage.getHeight() * clipping.height));

		//create the AffineTransform so that the path becomes relative to the smallImage
		transformToSmallPath = new AffineTransform();
		transformToSmallPath.setToTranslation(smallImageCoord.x , smallImageCoord.y);
		transformToSmallPath.scale(1 / (double)zoom, 1 / (double)zoom);

		//trace the equivalent subImage rectangle in the large image
		Rectangle largeImageCoord = new Rectangle((int)(largeImage.getWidth() * clipping.x),
				(int)(largeImage.getHeight() * clipping.y),
				(int)(largeImage.getWidth() * clipping.width),
				(int)(largeImage.getHeight() * clipping.height));

		//create the subImage from the larger image (for better quality)
		BufferedImage subImageLarge = largeImage.getSubimage(largeImageCoord.x,largeImageCoord.y,largeImageCoord.width,largeImageCoord.height);
		double ratio = ImageUtils.getResizingRatio(subImageLarge, smallImageCoord.width * zoom, smallImageCoord.height * zoom);
		subImage = resizeImage(subImageLarge, ratio, ratio);
		subImageLarge = null;
	}

	/**
	 * Returns the clipped image
	 * @return the clipped image
	 */
	public BufferedImage getSubImage() {
		return subImage;
	}

	/**
	 * Resize the given image to the desired width and height factors
	 * using the {@link AffineTransformOp#TYPE_BICUBIC} algorithm
	 * @param image the image to be resized
	 * @param widthFactor the desired width factor
	 * @param heightFactor the desired height factor
	 * @return a new resized image
	 */
	private BufferedImage resizeImage(BufferedImage image, double widthFactor, double heightFactor) {
		AffineTransform tx = new AffineTransform(); //use affine transform class to do the resizing
		tx.scale(widthFactor, heightFactor);
		AffineTransformOp op = new AffineTransformOp(tx, AffineTransformOp.TYPE_BICUBIC);
		return op.filter(image, null);
	}

	/**
	 * Converts a given path to it's equivalent relative to the small
	 * (not zoomed) image
	 * @param generalPath the path to convert
	 * @return a new transformed path
	 */
	public GeneralPath getSmallPath(GeneralPath generalPath) {
		return new GeneralPath(generalPath.createTransformedShape(transformToSmallPath));
	}
	
	public void dispose() {
		subImage = null;
	}
}