/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package utils;

import java.io.File;

import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.internal.gtk.OS;

//TODO support for ltr and rtl orientation icons should be implemented here
/**
 * This class provides easy access to the icons in use by the program.
 * The icons can be system stock icons or the program's stock icons
 */
public class StockIcon {

	/**
	 * Specifies the folder where the icons are located in the pixmaps folder
	 */
	private static final String STOCK_ICONS_FOLDER = "stock_icons";

	/**
	 * Specifies the folder where 16x16 sized icons are located in the stock icons folder
	 */
	private static final String STOCK_ICONS_16x16_FOLDER = "16x16";

	/**
	 * Specifies the folder where 22x22 sized icons are located in the stock icons folder
	 */
	private static final String STOCK_ICONS_22x22_FOLDER = "22x22";

	/**
	 * Specifies the folder where 24x24 sized icons are located in the stock icons folder
	 */
	private static final String STOCK_ICONS_24x24_FOLDER = "24x24";

	/**
	 * The size of a menu icon
	 */
	public static final int SIZE_MENU = 16;

	/**
	 * The size of a small tollbar or button icon
	 */
	public static final int SIZE_SMALL_TOOLBAR = 22;

	/**
	 * The size of a large toolbar or button icon
	 */
	public static final int SIZE_LARGE_TOOLBAR = 24;


	// All the icons available by the StockIcon
	public static final int ICON_HELP					= 1;
	public static final int ICON_ABOUT					= 2;
	public static final int ICON_OPEN					= 3;
	public static final int ICON_PREFERENCES			= 4;
	public static final int ICON_QUIT					= 5;
	public static final int ICON_FULLSCREEN				= 6;
	public static final int ICON_UNDO					= 7;
	public static final int ICON_REDO					= 8;
	public static final int ICON_CLEAR					= 9;
	public static final int ICON_SAVE					= 10;
	public static final int ICON_SAVE_AS				= 11;
	public static final int ICON_PRINT_PREVIEW			= 12;
	public static final int ICON_PRINT					= 13;
	public static final int ICON_CLOSE					= 14;
	public static final int ICON_CANCEL					= 15;
	public static final int ICON_OK						= 16;
	public static final int ICON_GO_UP					= 17;
	public static final int ICON_GO_BACK				= 18;
	public static final int ICON_GO_FORWARD				= 19;
	public static final int ICON_GO_DOWN				= 20;

	// The stock icon files that are associated to icon int constants
	private static final String STOCK_HELP				= "help.png";
	private static final String STOCK_ABOUT				= "about.png";
	private static final String STOCK_OPEN				= "document-open.png";
	private static final String STOCK_PREFERENCES		= "preferences.png";
	private static final String STOCK_QUIT				= "quit.png";
	private static final String STOCK_FULLSCREEN		= "fullscreen.png";
	private static final String STOCK_UNDO				= "undo.png";
	private static final String STOCK_REDO				= "redo.png";
	private static final String STOCK_CLEAR				= "clear.png";
	private static final String STOCK_SAVE				= "save.png";
	private static final String STOCK_SAVE_AS			= "save-as.png";
	private static final String STOCK_PRINT_PREVIEW		= "print-preview.png";
	private static final String STOCK_PRINT				= "print.png";
	private static final String STOCK_CLOSE				= "close.png";
	private static final String STOCK_CANCEL			= "cancel.png";
	private static final String STOCK_OK				= "ok.png";
	private static final String STOCK_GO_UP				= "go-up.png";
	private static final String STOCK_GO_BACK			= "go-back.png";
	private static final String STOCK_GO_FORWARD		= "go-forward.png";
	private static final String STOCK_GO_DOWN			= "go-down.png";

	/**
	 * Returns the matching stock image for the given constant image
	 * icon, which should be one of the icon constants specified in
	 * class <code>StockIcon</code>, the given constant image size,
	 * which should be one of the icon size constants specified in
	 * class <code>StockIcon</code>.
	 * This image should not be free'd because it could be that was
	 * allocated by the system, not the application.
	 * <p>
	 * If the allowOSStockIcon flag is set to true then the method tries
	 * to search for a matching system stock icon before returning the
	 * local stock icons folder
	 * </p>
	 *
	 * @param icon the StockIcon icon constant
	 * @param size the StockIcon icon size constant
	 * @param allowOSStockIcon whether to search for a matching system stock icon before returning the local stock icon
	 * @return the corresponding image data or <code>null</code> if match is not found
	 *
	 * @see For icon size constants use:
	 * @see StockIcon#SIZE_MENU
	 * @see StockIcon#SIZE_SMALL_TOOLBAR
	 * @see StockIcon#SIZE_LARGE_TOOLBAR
	 * 
	 */
	public static ImageData getIcon(int icon, int size, boolean allowOSStockIcon) {

		if (!allowOSStockIcon) { //check whether to use system icon or program's icon
			return getProgramStockIcon(icon, size);
		}

		//check for os name
		if (System.getProperty("os.name").equalsIgnoreCase("linux")) {
			
			//check for linux desktop name
			String desktopName = System.getProperty("sun.desktop");
			if (desktopName == null) {
				//unable to retrieve the desktop name, possible that sun.desktop system
				//variable is null. No problem, default icons will be used.
				desktopName = ""; //initialize the variable to avoid null exception
			}
			
			if (desktopName.equalsIgnoreCase("gnome")) {
				//try to get the GNOME stock icon
				ImageData gtkImageData = getGTKImageData(icon, size);
				if (gtkImageData != null) { //got icon from gtk
					return gtkImageData;
				} else { //failed to get gtk stock icon, use the icon from pixmaps
					return getProgramStockIcon(icon, size);
				}
			} else { //other linux desktops goes here (kde, etc..)
				return getProgramStockIcon(icon, size);
			}
		} else { //other operating systems goes here (Mac OS X, Windows, etc.)
			return getProgramStockIcon(icon, size);
		}
	}

	/**
	 * Returns the matching GTK stock image for the given constant
	 * image icon, which should be one of the icon constants specified
	 * in class <code>StockIcon</code>, the given constant image size,
	 * which should be one of the icon size constants specified in
	 * class <code>StockIcon</code>.
	 * This image should not be free'd because it was
	 * allocated by the system, not the application.
	 * 
	 * @param icon the StockIcon icon constant
	 * @param size the StockIcon icon size constant
	 * @return the corresponding image data or <code>null</code> if match is not found
	 */
	private static ImageData getGTKImageData(int icon, int size) {
		//set the equivalence with GtkStockIcon icons names
		String gtkIconName = null;
		switch (icon) {
		case ICON_HELP:
			gtkIconName = GtkStockIcon.GTK_STOCK_HELP;
			break;
		case ICON_ABOUT:
			gtkIconName = GtkStockIcon.GTK_STOCK_ABOUT;
			break;
		case ICON_OPEN:
			gtkIconName = GtkStockIcon.GTK_STOCK_OPEN;
			break;
		case ICON_PREFERENCES:
			gtkIconName = GtkStockIcon.GTK_STOCK_PREFERENCES;
			break;
		case ICON_QUIT:
			gtkIconName = GtkStockIcon.GTK_STOCK_QUIT;
			break;
		case ICON_FULLSCREEN:
			gtkIconName = GtkStockIcon.GTK_STOCK_FULLSCREEN;
			break;
		case ICON_UNDO:
			gtkIconName = GtkStockIcon.GTK_STOCK_UNDO;
			break;
		case ICON_REDO:
			gtkIconName = GtkStockIcon.GTK_STOCK_REDO;
			break;
		case ICON_CLEAR:
			gtkIconName = GtkStockIcon.GTK_STOCK_CLEAR;
			break;
		case ICON_SAVE:
			gtkIconName = GtkStockIcon.GTK_STOCK_SAVE;
			break;
		case ICON_SAVE_AS:
			gtkIconName = GtkStockIcon.GTK_STOCK_SAVE_AS;
			break;
		case ICON_PRINT_PREVIEW:
			gtkIconName = GtkStockIcon.GTK_STOCK_PRINT_PREVIEW;
			break;
		case ICON_PRINT:
			gtkIconName = GtkStockIcon.GTK_STOCK_PRINT;
			break;
		case ICON_CLOSE:
			gtkIconName = GtkStockIcon.GTK_STOCK_CLOSE;
			break;
		case ICON_CANCEL:
			gtkIconName = GtkStockIcon.GTK_STOCK_CANCEL;
			break;
		case ICON_OK:
			gtkIconName = GtkStockIcon.GTK_STOCK_OK;
			break;
		case ICON_GO_UP:
			gtkIconName = GtkStockIcon.GTK_STOCK_GO_UP;
			break;
		case ICON_GO_BACK:
			gtkIconName = GtkStockIcon.GTK_STOCK_GO_BACK;
			break;
		case ICON_GO_FORWARD:
			gtkIconName = GtkStockIcon.GTK_STOCK_GO_FORWARD;
			break;
		case ICON_GO_DOWN:
			gtkIconName = GtkStockIcon.GTK_STOCK_GO_DOWN;
			break;

		default:
			break;
		}

		//translate icon size into OS icon size
		int iconSizeGTK = -1;
		switch (size) {
		case SIZE_MENU:
			iconSizeGTK = OS.GTK_ICON_SIZE_MENU;
			break;
		case SIZE_SMALL_TOOLBAR:
			iconSizeGTK = OS.GTK_ICON_SIZE_SMALL_TOOLBAR;
			break;
		case SIZE_LARGE_TOOLBAR:
			iconSizeGTK = OS.GTK_ICON_SIZE_LARGE_TOOLBAR;
			break;

		default:
			break;
		}

		//ask GtkStockIcon for a matching icon with ltr orientation
		return  GtkStockIconSWT.getInstance().createImage(gtkIconName, iconSizeGTK, OS.GTK_TEXT_DIR_LTR);
	}


	//TODO do this statement apply to this method? if not who should free the image?
	// This image should not be free'd because it was
	// allocated by the system, not the application.
	/**
	 * Returns the matching program stock image for the given constant
	 * image icon, which should be one of the icon constants specified
	 * in class <code>StockIcon</code>, the given constant image size,
	 * which should be one of the icon size constants specified in
	 * class <code>StockIcon</code>.
	 * 
	 * @param icon the StockIcon icon constant
	 * @param size the StockIcon icon size constant
	 * @return the corresponding image data or <code>null</code> if icon file not found
	 */
	private static ImageData getProgramStockIcon(int icon, int size) {

		//set the equivalence with StockIcon icons names
		String stockIconName = null;
		switch (icon) {
		case ICON_HELP:
			stockIconName = StockIcon.STOCK_HELP;
			break;
		case ICON_ABOUT:
			stockIconName = StockIcon.STOCK_ABOUT;
			break;
		case ICON_OPEN:
			stockIconName = StockIcon.STOCK_OPEN;
			break;
		case ICON_PREFERENCES:
			stockIconName = StockIcon.STOCK_PREFERENCES;
			break;
		case ICON_QUIT:
			stockIconName = StockIcon.STOCK_QUIT;
			break;
		case ICON_FULLSCREEN:
			stockIconName = StockIcon.STOCK_FULLSCREEN;
			break;
		case ICON_UNDO:
			stockIconName = StockIcon.STOCK_UNDO;
			break;
		case ICON_REDO:
			stockIconName = StockIcon.STOCK_REDO;
			break;
		case ICON_CLEAR:
			stockIconName = StockIcon.STOCK_CLEAR;
			break;
		case ICON_SAVE:
			stockIconName = StockIcon.STOCK_SAVE;
			break;
		case ICON_SAVE_AS:
			stockIconName = StockIcon.STOCK_SAVE_AS;
			break;
		case ICON_PRINT_PREVIEW:
			stockIconName = StockIcon.STOCK_PRINT_PREVIEW;
			break;
		case ICON_PRINT:
			stockIconName = StockIcon.STOCK_PRINT;
			break;
		case ICON_CLOSE:
			stockIconName = StockIcon.STOCK_CLOSE;
			break;
		case ICON_CANCEL:
			stockIconName = StockIcon.STOCK_CANCEL;
			break;
		case ICON_OK:
			stockIconName = StockIcon.STOCK_OK;
			break;
		case ICON_GO_UP:
			stockIconName = StockIcon.STOCK_GO_UP;
			break;
		case ICON_GO_BACK:
			stockIconName = StockIcon.STOCK_GO_BACK;
			break;
		case ICON_GO_FORWARD:
			stockIconName = StockIcon.STOCK_GO_FORWARD;
			break;
		case ICON_GO_DOWN:
			stockIconName = StockIcon.STOCK_GO_DOWN;
			break;

		default:
			break;
		}

		//translate icon size into folder name
		String sizeFolder = null;
		switch (size) {
		case SIZE_MENU:
			sizeFolder = STOCK_ICONS_16x16_FOLDER;
			break;
		case SIZE_SMALL_TOOLBAR:
			sizeFolder = STOCK_ICONS_22x22_FOLDER;
			break;
		case SIZE_LARGE_TOOLBAR:
			sizeFolder = STOCK_ICONS_24x24_FOLDER;
			break;

		default:
			break;
		}

		try {
			//create a image data with the icon read from the pixmaps folder
			return new ImageData(
					System.getProperty("program.pixmapspath") +
					File.separator + STOCK_ICONS_FOLDER +
					File.separator + sizeFolder +
					File.separator + stockIconName
			);
		} catch (Exception e) {
			return null;
		}
	}
}